@@ -77,8 +77,9 @@
 :- autoload(library(prolog_stream),[open_prolog_stream/4]).
 :- autoload(library(readutil),[read_line_to_string/2]).
 :- autoload(library(yall),[(>>)/4]).
-:- autoload(library(http/html_write),[html/3,print_html/1]).
 :- autoload(library(http/term_html),[term/4]).
+
+:- use_module(library(http/html_write),[html/3,print_html/1, op(_,_,_)]).
 :- use_module(library(settings),[setting/4,setting/2]).
 
 :- use_module(library(sandbox), []).
@@ -528,9 +529,16 @@ clauses_html(Clauses, _ID, HTMLString) :-
 
 list_clauses([]).
 list_clauses([H|T]) :-
-    portray_clause(H),
+    (   system_undefined(H)
+    ->  true
+    ;   portray_clause(H)
+    ),
     list_clauses(T).
 
+system_undefined((undefined :- tnot(undefined))).
+system_undefined((answer_count_restraint :- tnot(answer_count_restraint))).
+system_undefined((radial_restraint :- tnot(radial_restraint))).
+
 dict_bindings(Dict, Bindings) :-
     dict_pairs(Dict, _Tag, Pairs),
     maplist([N-V,N=V]>>true, Pairs, Bindings).
