@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2013-2018, University of Amsterdam
+    Copyright (c)  2013-2020, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
     All rights reserved.
@@ -38,18 +38,6 @@
           [ test_ssl/0
           ]).
 
-:- asserta(user:file_search_path(library, '.')).
-:- asserta(user:file_search_path(library, '../clib')).
-:- asserta(user:file_search_path(library, '..')).
-:- asserta(user:file_search_path(library, '../plunit')).
-:- asserta(user:file_search_path(library, '../sgml')).
-:- asserta(user:file_search_path(foreign, '../clib')).
-:- asserta(user:file_search_path(foreign, '.')).
-:- asserta(user:file_search_path(foreign, '../http')).
-:- asserta(user:file_search_path(foreign, '../sgml')).
-:- prolog_load_context(directory, D),
-   asserta(user:file_search_path(library, D)).
-
 :- use_module(library(plunit)).
 :- use_module(library(ssl)).
 :- use_module(library(crypto)).
@@ -57,7 +45,7 @@
 :- use_module(library(error)).
 :- use_module(library(readutil)).
 :- use_module(library(socket)).
-:- use_module(library(https)).
+:- use_module(https).
 
 %:- debug(connection).
 %:- debug(certificate).
@@ -238,6 +226,19 @@ test(missing_key, Msg == 'no private key assigned') :-
 
 :- begin_tests(ssl_server).
 
+% This test creates a server and client and tests the following
+% sequence:
+%
+%   - Client sends "Hello world" and reads the reply with a 1 sec
+%     timeout.
+%   - Server reads "Hello world", _waits_ 1.5 sec before echoing it.
+%   - Client gets a timeout and retries the read.  Second try should
+%     read "Hello world"
+%   - Client sends "bye", going through the same cycle as above.
+%     (JW: Why a second time?)
+%   - Client closes connection.
+%   - Server reads 'end_of_file' and closes the connection.
+
 test(server) :-
     make_server(SSL, Socket),
     thread_create(server_loop(SSL, Socket), Id, []),
@@ -247,6 +248,7 @@ test(server) :-
             report_join_status(Status)
         ;   format(user_error, 'Client error:~n', []),
             print_message(error, E),
+            thread_signal(Id, abort),
             thread_join(Id, Status),
             report_join_status(Status),
             fail
@@ -254,6 +256,7 @@ test(server) :-
     ).
 
 report_join_status(true).
+report_join_status('$aborted').                 % we killed the server.
 report_join_status(false) :-
     print_message(error, goal_failed(server_loop(_))).
 report_join_status(exception(Term)) :-
@@ -286,10 +289,8 @@ make_server(SSL, Socket) :-
                   certificate_file('tests/test_certs/server-cert.pem'),
                   key_file('tests/test_certs/server-key.pem'),
                   cert_verify_hook(get_cert_verify),
-%                     password('apenoot1'),
                   pem_password_hook(get_server_pwd)
                 ]),
-%   Port = 1111,
     tcp_socket(Socket),
     tcp_setopt(Socket, reuseaddr),
     tcp_bind(Socket, localhost:Port),
@@ -318,9 +319,9 @@ copy_client(In, Out) :-
     read_line_to_codes(In, Line),
     (   Line == end_of_file
     ->  true
-    ;   debug(data, 'SERVER: Got ~s~n', [Line]),
+    ;   debug(data, 'SERVER: Got ~s (sleeping 1.5 sec)', [Line]),
         sleep(1.5),
-        debug(data, 'SERVER: writing ~s~n', [Line]),
+        debug(data, 'SERVER: writing ~s', [Line]),
         format(Out, '~s~n', [Line]),
         flush_output(Out),
         (   atom_codes(bye, Line)
@@ -361,7 +362,6 @@ client :-
                certificate_file('tests/test_certs/client-cert.pem'),
                key_file('tests/test_certs/client-key.pem'),
                close_parent(true),
-%                  password('apenoot2'),
                pem_password_hook(get_client_pwd)
              ]),
     client_loop(SSL).
@@ -383,25 +383,26 @@ client_loop(SSL) :-
     call_cleanup(close(In), close(Out)).
 
 write_server(Message, In, Out) :-
-    debug(data, 'CLIENT: writing: ~q~n', [Message]),
+    debug(data, 'CLIENT: writing: ~q', [Message]),
     write(Out, Message), nl(Out),
     flush_output(Out),
     sleep(0.1),
+    debug(data, 'CLIENT: attempting to read reply (timeout 1 sec)', []),
     catch(read_from_server(In, Message),
           E,
-          debug(data, 'CLIENT: exception: ~q~n', [E])),
+          debug(data, 'CLIENT: exception: ~q', [E])),
     (   var(E)
     ->  true
-    ;   read_from_server(In, Message)
+    ;   debug(data, 'CLIENT: retrying read reply (timeout 1 sec)', []),
+        read_from_server(In, Message)
     ).
 
 read_from_server(In, Message) :-
-    debug(data, 'CLIENT: attempting to read reply from stream~n', []),
     read_line_to_codes(In, Line),
     (   Line == end_of_file
     ->  true
     ;   atom_codes(Reply, Line),
-        debug(data, 'CLIENT: Got ~q~n', [Reply]),
+        debug(data, 'CLIENT: Got ~q', [Reply]),
         (   Reply == Message
         ->  true
         ;   format(user_error, 'CLIENT: ERROR: Sent ~q, Got ~q~n',
