@@ -34,6 +34,7 @@
 */
 
 #include "pl-incl.h"
+#include "pl-comp.h"
 #include "pl-rsort.h"
 #include <math.h>
 
@@ -1058,12 +1059,6 @@ gcClauseList(ClauseList clist, DirtyDefInfo ddi, gen_t start)
     }
   }
 
-  DEBUG(CHK_SECURE,
-	{ for(cref=clist->first_clause; cref; cref=cref->next)
-	  { assert(false(cref->value.clause, CL_ERASED));
-	  }
-	});
-
   clist->erased_clauses = 0; /* see (*) */
 }
 
@@ -2169,7 +2164,7 @@ compar_keys(const void *p1, const void *p2)
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Only the final call gets a clause_count > 0. Here we do the remainder of
-the assessment. We could consider  for   a  seperate  function to merely
+the assessment. We could consider  for   a  separate  function to merely
 reduce the set.
 
 (*) Currently we cannot  combine  variables   with  functor  indexes  as
